/* ****************************************************************************
  This file is part of KBabel

  Copyright (C) 1999-2000 by Matthias Kiefer
                            <matthias.kiefer@gmx.de>
                2001-2004 by Stanislav Visnovsky
                            <visnovsky@kde.org>
                2002-2003 by Marco Wegner 
                            <mail@marcowegner.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  In addition, as a special exception, the copyright holders give
  permission to link the code of this program with any edition of
  the TQt library by Trolltech AS, Norway (or with modified versions
  of TQt that use the same license as TQt), and distribute linked
  combinations including the two.  You must obey the GNU General
  Public License in all respects for all of the code used other than
  TQt. If you modify this file, you may extend this exception to
  your version of the file, but you are not obligated to do so.  If
  you do not wish to do so, delete this exception statement from
  your version.

**************************************************************************** */


#include <tqfile.h>
#include "tqregexp.h"
#include <tqstring.h>
#include <tqstringlist.h>
#include <tqtextstream.h>

#include <kgenericfactory.h>

#include "catalog.h"
#include "catalogitem.h"
#include "catalogsettings.h"
#include "xliffexport.h"

K_EXPORT_COMPONENT_FACTORY( kbabel_xliffexport, KGenericFactory<XLIFFExportPlugin> ( "kbabelxliffexportfilter" ) )

using namespace KBabel;

XLIFFExportPlugin::XLIFFExportPlugin( TQObject * parent, const char * name, const TQStringList& )
  : CatalogExportPlugin( parent, name )
{
}

ConversionStatus XLIFFExportPlugin::save( const TQString& filename, const TQString&, const Catalog * catalog )
{
  // Check whether we know how to handle the extra data.
  if ( catalog->importPluginID( ) != "XLIFF 1.1" )
    return UNSUPPORTED_TYPE;
  
  TQFile file( filename );
  if ( !file.open( IO_WriteOnly ) )
    return OS_ERROR;
    
  SaveSettings settings = catalog->saveSettings( );

  // New DOM document.  
  TQDomDocument doc( "" );
  
  extraData = catalog->catalogExtraData();
  kdDebug () << "Setting the document data: " << extraData.first () << endl;  
  doc.setContent ( extraData.first () );

  // Regular messages.
  for ( uint i = 0; i < catalog->numberOfEntries( ); i++ ) {
    TQDomElement element = extractComment( doc, *(extraData.at( i+1 )) );
    createMessage( doc, element, catalog->msgid( i ).join( "" ), catalog->msgstr( i ).join( "" ) );
  }
  
  TQTextStream stream( &file );
  doc.save( stream, 2 );
  file.close( );
  
  return OK;
}

TQDomElement XLIFFExportPlugin::extractComment( TQDomDocument& doc, const TQString& s )
{
    TQString comment( s );
  
    if ( comment.isEmpty () )
    {
	kdError () << "Empty comment, should not happen" << endl;
    }
  
    // Extract the context and the actual comment.
    comment.remove( TQRegExp( "^Context:[\\s]*" ) );
    TQString newContext;
    TQStringList commentlines = TQStringList::split ( '\n', comment);
    
    TQString file = *(commentlines.at(0));
    TQString id = *(commentlines.at(1));
    
    kdDebug () << "Looking for file " << file << endl;

    return getContext( doc, file, id );
}

void XLIFFExportPlugin::createMessage( TQDomDocument& doc, TQDomElement& translationElement, const TQString& msgid, 
     const TQString& msgstr )
{
    // for empty messages, don't store anything
    if (msgstr.isEmpty ())
	return;
	
    // find the trans element
    TQDomNode node = translationElement.firstChild( );
    while ( !node.isNull( ) ) {
	kdDebug () << node.nodeName () << endl;
	if ( node.isElement()  && node.toElement().tagName( ) == "target") {
	    kdDebug () << "Found the target: " << 
	    node.firstChild().nodeName () << endl;
	    // set the new translation
	    node.firstChild().toText().setData (msgstr);
	    break;
	}
	node = node.nextSibling( );
    }

    if ( node.isNull () )
    {
	// no target tag, create one
	node = doc.createElement ("target");
	translationElement.appendChild (node);
	
	TQDomText data = doc.createTextNode(msgstr );
	node.appendChild( data );
    }    
}

TQDomElement XLIFFExportPlugin::getContext( TQDomDocument& doc, const TQString& file, const TQString& id )
{
  // Find out whether there is already such a context in the TQDomDocument.
  TQDomNode parentelem = doc.documentElement();
  TQDomNode elem = doc.documentElement( ).firstChild( );
  while ( !elem.isNull( ) ) {
      if ( elem.isElement( ) && elem.toElement().tagName( ) == "file" && elem.toElement().attribute ("original") == file ) {
        kdDebug () << "We have found the file" << endl;
	break;
      }
      elem = elem.nextSibling( );
  }
  
  if (elem.isNull ())
  {
      kdError () << "File not found at all, creating" << endl;
      TQDomElement newelem = doc.createElement ("file");
      newelem.setAttribute ("original", file);
      parentelem.appendChild (newelem);
      elem = newelem;
  }
  
  // lookup body tag
  parentelem = elem;
  elem = elem.firstChild ();
  while ( !elem.isNull( ) ) {
      if ( elem.isElement( ) && elem.toElement().tagName( ) == "body" ) {
        kdDebug () << "We have found the file body" << endl;
	break;
      }
      elem = elem.nextSibling( );
  }
  
  if (elem.isNull ())
  {
      kdError () << "File body not found at all, creating" << endl;
      TQDomElement newelem = doc.createElement ("body");
      parentelem.appendChild (newelem);
      elem = newelem;
  }
  
  elem = findTransUnit (elem, id);
  
  if (elem.isNull ())
  {
      kdError () << "Trans-unit not found at all, creating" << endl;
      TQDomElement newelem = doc.createElement ("trans-unit");
      newelem.setAttribute ("id", id);
      parentelem.appendChild (newelem);
      elem = newelem;
  }
  
  return elem.toElement ();
}

TQDomElement XLIFFExportPlugin::findTransUnit( TQDomNode& group, const TQString& id )
{
  TQDomNode elem = group.firstChild( );

  // lookup correct trans-unit tag
  while ( !elem.isNull( ) ) {
      if ( elem.isElement( ) && elem.toElement().tagName() == "group" )
      {
        // search recursively
	TQDomElement res = findTransUnit( elem, id );
	if (! res.isNull () )
	    return res.toElement();
      }
      else if ( elem.isElement( ) && elem.toElement().tagName( ) == "trans-unit" && elem.toElement().attribute ("id") == id ) {
        kdDebug () << "We have found the trans-unit" << endl;
	return elem.toElement ();
      }
      elem = elem.nextSibling( );
  }
  
  return elem.toElement ();
}
